package model;


import java.awt.Graphics;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

public class Edge {
	/**
	 * Matice incidence
	 */
	private Map<String, Map<String, Integer>> matrix = new HashMap<String, Map<String, Integer>>();
	private Map<String, Integer> vzdalenost = new HashMap<String, Integer>();
	Graphics g;

	public Edge(Graphics g) {
		this.g = g;
	}

	private void addHrana(String novaHrana) {

		for (Map<String, Integer> row : matrix.values()) {
			row.put(novaHrana, 0);

		}

		Map<String, Integer> list = new HashMap<String, Integer>();
		for (String n : matrix.keySet()) {
			list.put(n, 0);
		}
		list.put(novaHrana, 0);

		matrix.put(novaHrana, list);

	}

	protected void removeNode(String node) {
		matrix.remove(node);
		for (Map<String, Integer> row : matrix.values()) {
			row.remove(node);
		}
	}

	public Set<String> getNodes() {

		return matrix.keySet();
	}

	public List<String> findPath(String prvni, String druhy) {
		for (String node : getNodes()) {
			vzdalenost.put(node, Integer.MAX_VALUE);
		}
		vzdalenost.put(prvni, 0);

		Map<String, String> predecessors = new HashMap<String, String>();
		String currentNode = null;

		while ((currentNode = findNearestNode()) != null) {
			for (String adjacentNode : findAdjacentNodes(currentNode)) {
				int possiblyBetterDistance = vzdalenost.get(currentNode)
						+ distance(currentNode, adjacentNode);
				if (possiblyBetterDistance < vzdalenost.get(adjacentNode)) {
					vzdalenost.put(adjacentNode, possiblyBetterDistance);
					predecessors.put(adjacentNode, currentNode);
				}
			}

			removeNode(currentNode);
			vzdalenost.remove(currentNode);

		}

		List<String> path = new ArrayList<String>();
		String predecessor;
		path.add(druhy);
		String node = druhy;

		while ((predecessor = predecessors.get(node)) != null) {
			path.add(0, predecessor);
			node = predecessor;

		}

		return path;
	}

	private String findNearestNode() {
		int minimum = Integer.MAX_VALUE;
		String nearestNode = null;
		for (Entry<String, Integer> entry : vzdalenost.entrySet()) {
			if (entry.getValue() < minimum) {
				minimum = entry.getValue();
				nearestNode = entry.getKey();

			}
		}

		return nearestNode;
	}

	private int distance(String node1, String node2) {
		return matrix.get(node1).get(node2);
	}

	private Set<String> findAdjacentNodes(String node) {
		Set<String> adjacentNodes = new HashSet<String>();
		for (Entry<String, Integer> item : matrix.get(node).entrySet()) {
			if (item.getValue() > 0) {
				adjacentNodes.add(item.getKey());

			}

		}

		return adjacentNodes;
	}

	public void drawEdge(String startname, String endName, int weight) {

		if (!matrix.containsKey(startname)) {
			addHrana(startname);

		}
		if (!matrix.containsKey(endName)) {
			addHrana(endName);

		}

		matrix.get(startname).put(endName, weight);
		matrix.get(endName).put(startname, weight);

	}

}
